var Comet = Comet || {};

Comet.Modal = function() {
    'use strict';

    var $overlay,
        $modal,
        $closeButton,
        $lastFocus;

    var openInModal = function(target) {
        // target: thing to be opened in modal
        var $content = $('#' + target).clone().attr('id', target + "-clone");

        // fire custom event
        $('body').trigger('comet.modal.begin');
        // add class to body to identify a modal is open
        $('body').addClass('comet-modal-open');
        // add overlay to page

        generateModal($content);
        attachCloseEvents();
    };

    var generateModal = function(content) {
        // Create modal base objects
        var $modaloverlay = $('<div class="comet-modal-overlay comet-modal-overlay--backdrop js-comet-modal-overlay" role="dialog"></div>'),
            $modalContainer = $('<div class="comet-modal js-comet-modal" role="document"></div>');
        // insert content into modal container
        content.appendTo($modalContainer);
        // add modal and overlay to page
        $('body').append($modaloverlay);
        $('body').append($modalContainer);

        // resize modal in order to account for smaller screens for scrollable
        resizeModal();

        // define $overlay, $modal and $closeButton vars for closeModal()
        $overlay = $('.js-comet-modal-overlay');
        $modal = $('.js-comet-modal');
        $closeButton = $('.js-comet-modal__close-button');

        $modal.attr('tabindex', '0');
        $modal.focus();

        // fire custom event
        $('body').trigger('comet.modal.generate');
    };

    var attachCloseEvents = function() {
        $overlay.on('click', closeModal);
        $closeButton.on('click', closeModal);
        // Esc Key
        $(document).keyup(function(e) {
            if (e.keyCode === 27) {
                closeModal();
            }
        });
    };

    var attachOpenEvents = function() {

        $('.js-modal-trigger').on('click', function() {
            var $modalTarget = $(this).data('target');
            $lastFocus = document.activeElement;
            openInModal($modalTarget);
        });
    };

    var closeModal = function() {
        $overlay.remove();
        $modal.remove();
        $lastFocus.focus();
        $('body').removeClass('comet-modal-open');

        // fire custom event
        $('body').trigger('comet.modal.close');
    };

    var resizeModal = function() {
        var $resizeModal = $('.js-comet-modal'),
            $modalInner = $resizeModal.find('.comet-modal__inner'),
            $modalContent = $resizeModal.find('.comet-modal__content'),
            $modalHeaderHeight = $resizeModal.find('.comet-modal__header').height(),
            $modalFooterHeight = $resizeModal.find('.comet-modal__footer').outerHeight(),
            availableHeight = document.body.clientHeight,
            $newHeight;

        $newHeight = availableHeight - ($modalHeaderHeight + $modalFooterHeight + 80); // 80 is for misc spacing and margins
        if ($newHeight > 775) { $newHeight = 775; }
        $modalContent.css('max-height', $newHeight);

        if (Comet.Utils.hasScrollBar($modalContent)) {
            $modalInner.addClass("comet-modal--scrollable");
        } else {
            $modalInner.removeClass("comet-modal--scrollable");
        }

    };

    var initialize = function() {
        attachOpenEvents();

        $(window).resize(function() {
            if ($('.js-comet-modal').length > 0) {
                Comet.Utils.debounceFunction(resizeModal(), 200);
            }
        });
    };

    var public_vars = {
        'initialize': initialize,
        'openInModal': openInModal,
        'generateModal': generateModal,
        'closeModal': closeModal,
        'attachOpenEvents': attachOpenEvents,
        'attachCloseEvents': attachCloseEvents
    };

    return public_vars;
}();

$(document).ready(function() {
    "use strict";

    Comet.Modal.initialize();
});
